#include "gps.h"
#include "r1.h"

void user_info(int LongDeg, int LongMin, int LatDeg, int LatMin, double Alt, double user_pos[]){

	double Long;
	double Lat;

	double Pos[3]; /* postion longitude, lattitude, altitude */


    Long = LongDeg + (LongMin / 60.0);
    Lat  = LatDeg  + (LatMin  / 60.0);
 //    printf( "Enter altitude in meters : \n" );
 //    scanf( "%le", &Alt );
	Pos[0] = Lat;
	Pos[1] = Long;
	Pos[2] = Alt;

	//    ConvertToECEF( user_pos, Pos );
	LLA2XYZ( user_pos, Pos );
	printf( "Pos in earth center fixed coord %le %le %le\n",
				 user_pos[0], user_pos[1], user_pos[2] );

	//    ConvertToLongLatAlt( Pos, user_pos );
	XYZ2LLA( Pos, user_pos );
	printf( "Lat %le long %le alt %le\n", Pos[0], Pos[1], Pos[2] );
}

void XYZ2LLA( double Xo[], double Xi[]){
  /*ECEF X [m], Y [m] , Z[m]*/
  /*lat [rad], lon [rad] alt [m]*/
  /*this procedure converts WGS-84 ECEF XYZ to Lat, Lon, Alt [above ellipsoid]*/
  double p, T, sT, cT, N;
  
  p = sqrt(Xi[0] * Xi[0] + Xi[1] * Xi[1]);
  T = atan(Xi[2] * a / (p * b));
  sT = sin(T);
  cT = cos(T);
  Xo[0] = atan((Xi[2] + e2sqr * b * sT * sT * sT) / (p - e1sqr * a * cT * cT * cT));
  if (Xi[0] == 0.0)
    Xo[1] = PI / 2.0;
  else
    Xo[1] = atan(Xi[1] / Xi[0]);
  N = a / sqrt(1.0 - e1sqr * sin(Xo[0]) * sin(Xo[0]));
  Xo[2] = p / cos(Xo[0]) - N;
  /* degree conversion*/
  Xo[0] *= 180.0 / PI;
  Xo[1] *= 180.0 / PI;
}

void LLA2XYZ( double *Xo, double *Xi){
  /*lat [rad], lon [rad] alt [m]*/
  /*ECEF X [m], Y [m] , Z[m]*/
  /*this procedure converts WGS-84 Lat, Lon and Alt [above ellipsoid]
    to ECEF XYZ*/
  double N;
  
  N = a / sqrt(1.0 - e1sqr * sin(Xi[0] * PI / 180.0) * sin(Xi[0] * PI / 180.0));
  
  Xo[0] = (N + Xi[2]) * cos(Xi[0] * PI / 180.0) * cos(Xi[1] * PI / 180.0);
  Xo[1] = (N + Xi[2]) * cos(Xi[0] * PI / 180.0) * sin(Xi[1] * PI / 180.0);
  Xo[2] = (N * (1.0 - e1sqr) + Xi[2]) * sin(Xi[0] * PI / 180.0);
}

void calc_azimel(position_sat_t *sat_gps_ptr, double user_pos[])
{
  /* IN: 	satellite ECEF XYZ*/
  /* IN: 	user ECEF XYZ*/
  /* OUT: azimuth [rad]*/
  /* OUT: elevation [rad]*/
  /* OUT: calculation succeeded: stat = true*/
  double R, p, x, y, z, s;
  double e[3][3];
  long i, k;
  double d[3];
  double Xs[3],Xu[3];
  
  Xs[0]=sat_gps_ptr->x;
  Xs[1]=sat_gps_ptr->y;
  Xs[2]=sat_gps_ptr->z;
  
  Xu[0] = user_pos[0];
  Xu[1] = user_pos[1];
  Xu[2] = user_pos[2];
  
  x = user_pos[0];
  y = user_pos[1];
  z = user_pos[2];
  p = sqrt(x * x + y * y);

  R = sqrt(x * x + y * y + z * z);
  e[0][0] = -(y / p);
  e[0][1] = x / p;
  e[0][2] = 0.0;
  e[1][0] = -(x * z / (p * R));
  e[1][1] = -(y * z / (p * R));
  e[1][2] = p / R;
  e[2][0] = x / R;
  e[2][1] = y / R;
  e[2][2] = z / R;
  
  /* matrix multiply vector from user */
  for (k = 0; k <= 2; k++) {
		d[k] = 0.0;
		for (i = 0; i <= 2; i++) {
			d[k] += (Xs[i] - Xu[i]) * e[k][i];
		}
	}
  
  s = d[2] / sqrt(d[0] * d[0] + d[1] * d[1] + d[2] * d[2]);
  if (s == 1.0) {
		sat_gps_ptr->elevation = 0.5 * PI;
	} else {
		sat_gps_ptr->elevation = atan(s / sqrt(1.0 - s * s));
	}
  
  if (d[1] == 0.0 && d[0] > 0.0) {
		sat_gps_ptr->azimuth = 0.5 * PI;
	} else if (d[1] == 0.0 && d[0] < 0.0) {
		sat_gps_ptr->azimuth = 1.5 * PI;
	} else {
		sat_gps_ptr->azimuth = atan(d[0] / d[1]);
		if (d[1] < 0.0) {
			sat_gps_ptr->azimuth += PI;
		} else if (d[1] > 0.0 && d[0] < 0.0) {
			sat_gps_ptr->azimuth += 2.0 * PI;
		}
	}
}  /*calc_azimel*/
